/*
 * Copyright (c) 2016 Chris Johns <chrisj@rtems.org>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#include <stdlib.h>
#include <sys/stat.h>

#include <rtems.h>
#include <rtems/shell.h>
#include <rtems/telnetd.h>
#include <rtems/tinysshd.h>

#define TEST_NAME "LIBBSD TINYSSH 1"

/*
 * Cyrpto tests.
 */
int crypto_verify_16test__main(void);
int crypto_verify_32test__main(void);
int crypto_auth_hmacsha256test__main(void);
int crypto_core_aes256encrypttest__main(void);
int crypto_hash_sha256test__main(void);
int crypto_hash_sha512test__main(void);
int crypto_onetimeauth_poly1305test__main(void);
int crypto_scalarmult_curve25519test__main(void);
int crypto_sign_ed25519test__main(void);
int crypto_sign_nistp256ecdsatest__main(void);
int crypto_stream_chacha20test__main(void);

void
cyrpto_tests(void)
{
  printf("CRYPTO TESTS: start\n");
    printf("  crypto_verify_16test\n");
    crypto_verify_16test__main();
    printf("  crypto_verify_32test\n");
    crypto_verify_32test__main();
    printf("  crypto_auth_hmacsha256test\n");
    crypto_auth_hmacsha256test__main();
    printf("  crypto_core_aes256encrypttest\n");
    crypto_core_aes256encrypttest__main();
    printf("  crypto_hash_sha256test\n");
    crypto_hash_sha256test__main();
    printf("  crypto_hash_sha512test\n");
    crypto_hash_sha512test__main();
    printf("  crypto_onetimeauth_poly1305test\n");
    crypto_onetimeauth_poly1305test__main();
    printf("  crypto_scalarmult_curve25519test\n");
    crypto_scalarmult_curve25519test__main();
    printf("  crypto_scalarmult_nistp256test\n");
    crypto_scalarmult_nistp256test__main();
    printf("  crypto_sign_ed25519test\n");
    crypto_sign_ed25519test__main();
    printf("  crypto_sign_nistp256ecdsatest\n");
    crypto_sign_nistp256ecdsatest__main();
    printf("  crypto_stream_chacha20testn");
    crypto_stream_chacha20test__main();
  printf("CRYPTO TESTS: end\n");
}

int initgroups(const char *name, gid_t basegid)
{
  return 0;
}

void randombytes(unsigned char *x,unsigned long long xlen)
{
  int offset = random() & 3;
  while (xlen-- > 0)
  {
    long r = random();
    *x++ = (unsigned char) ((r >> (offset * 8)) & 0xff);
    offset = r & 3;
  }
}

static void
test_main(void)
{
  cyrpto_tests();
  rtems_task_delete(RTEMS_SELF);
}

#define CONFIGURE_MAXIMUM_DRIVERS 32

#include <rtems/bsd/test/default-network-init.h>
